#!/usr/bin/env python
#
# Copyright (c) 2024, Marvell.
#
# SPDX-License-Identifier: MIT
#
# These scripts can be used for parsing and verifying LiquidSecurity HSM
# adapter attestation data; you may redistribute it and/or modify it under
# the terms of the MIT license.
#
import sys
import base64
import hashlib
"""
	Description: This script compares the KCV and eKCV values in the extracted public key to  
	the values in the parsed attestation data. If they match, the verification 	is successful.

      Args:
        parse.dat: The parsed attestation data..
        public_key.pem: The public key file

      Returns:
        True if public key matches attestation values; false if values do not match.
		
	Example: python verify_pubkey.py parse.dat public_key.pem
"""
public_key_file = sys.argv.pop()
parsed_attest_file = sys.argv.pop()
with open(public_key_file, 'r') as public_key_data:
  publickey = "".join(public_key_data.read().split('\n')[1:-2])
  publickey = base64.b64decode(publickey)
  # KCV is the first 6 bytes of the SHA1 digest of the key.
  kcv = hashlib.sha1(publickey).hexdigest()[:6]
  # EKCV is the SHA256 digest of the key
  ekcv = hashlib.sha256(publickey).hexdigest()

  with open(parsed_attest_file, 'r') as parsed_attest_data:
    attest_data = parsed_attest_data.read()
    if kcv in attest_data:
      if '0x1003:' not in attest_data:
        print('EKCV not present in attestation data. Skipping EKCV check.')
        print('Public key matched with attestation based on KCV.')
      elif ekcv in attest_data:
        print('Public key matched with attestation.')
      else:
        print('Public key not matched with attestation.')
    else:
      print('Public key not matched with attestation.')
      attest_lines = attest_data.split('\n')
      # field 0x0173 corresponds to KCV,
      # field 1003 (Custom Cavium) corresponds to EKCV
      kcv_lines = [line for line in attest_lines if line.startswith('0x0173:') or line.startswith('0x1003:')]
      print("KCV in attestation: %s" %(kcv_lines))
      print(kcv)
      print(ekcv)
